@@ -24,6 +24,10 @@
 #include "collectionstatistics.h"
 #include "collection_p.h"
 
+#include <akonadi/private/protocol_p.h>
+
+#include <KLocale>
+
 using namespace Akonadi;
 
 class Akonadi::CollectionModifyJobPrivate : public JobPrivate
@@ -51,26 +55,24 @@ CollectionModifyJob::~CollectionModifyJob()
 void CollectionModifyJob::doStart()
 {
   Q_D( CollectionModifyJob );
-  if ( !d->mCollection.isValid() && d->mCollection.remoteId().isEmpty() ) {
-    setError( Unknown );
-    setErrorText( QLatin1String( "Invalid collection" ) );
+  QByteArray command = d->newTag();
+  try {
+    command += ProtocolHelper::entityIdToByteArray( d->mCollection, AKONADI_CMD_COLLECTIONMODIFY );
+  } catch ( const std::exception &e ) {
+    setError( Job::Unknown );
+    setErrorText( QString::fromUtf8( e.what() ) );
     emitResult();
     return;
   }
 
-  QByteArray command = d->newTag();
-  if ( d->mCollection.isValid() )
-    command += " MODIFY " + QByteArray::number( d->mCollection.id() );
-  else
-    command += " RID MODIFY " + ImapParser::quote( d->mCollection.remoteId().toUtf8() );
   QByteArray changes;
   if ( d->mCollection.d_func()->contentTypesChanged ) {
     QList<QByteArray> bList;
     foreach( const QString &s, d->mCollection.contentMimeTypes() ) bList << s.toLatin1();
     changes += " MIMETYPE (" + ImapParser::join( bList, " " ) + ')';
   }
-  if ( d->mCollection.parent() >= 0 )
-    changes += " PARENT " + QByteArray::number( d->mCollection.parent() );
+  if ( d->mCollection.parentCollection().id() >= 0 )
+    changes += " PARENT " + QByteArray::number( d->mCollection.parentCollection().id() );
   if ( !d->mCollection.name().isEmpty() )
     changes += " NAME " + ImapParser::quote( d->mCollection.name().toUtf8() );
   if ( !d->mCollection.remoteId().isNull() )
@@ -89,4 +91,10 @@ void CollectionModifyJob::doStart()
   d->writeData( command );
 }
 
+Collection CollectionModifyJob::collection() const
+{
+  const Q_D( CollectionModifyJob );
+  return d->mCollection;
+}
+
 #include "collectionmodifyjob.moc"
