@@ -49,9 +49,8 @@ using namespace Akonadi;
 struct ItemContainer
 {
   ItemContainer( const Item& i, int r )
+    : item( i ), row( r )
   {
-    item = i;
-    row = r;
   }
   Item item;
   int row;
@@ -117,7 +116,7 @@ bool ItemModel::Private::collectionIsCompatible() const
     return true;
   // if the model's mime types are more specific, limit to those
   // collections that have matching types
-  Q_FOREACH( QString type, mParent->mimeTypes() ) {
+  Q_FOREACH( const QString &type, mParent->mimeTypes() ) {
     if ( collection.contentMimeTypes().contains( type ) ) {
       return true;
     }
@@ -131,20 +130,24 @@ void ItemModel::Private::listingDone( KJob * job )
   Q_UNUSED( fetch );
   if ( job->error() ) {
     // TODO
-    kWarning( 5250 ) << "Item query failed:" << job->errorString();
+    kWarning() << "Item query failed:" << job->errorString();
   }
 }
 
 void ItemModel::Private::collectionFetchResult( KJob * job )
 {
   CollectionFetchJob *fetch = static_cast<CollectionFetchJob*>( job );
+
+  if ( fetch->collections().isEmpty() )
+    return;
+
   Q_ASSERT( fetch->collections().count() == 1 ); // we only listed base
   Collection c = fetch->collections().first();
    // avoid recursion, if this fails for some reason
   if ( !c.contentMimeTypes().isEmpty() ) {
     mParent->setCollection(c);
   } else {
-    kWarning( 5250 ) << "Failed to retrieve the contents mime type of the collection: " << c;
+    kWarning() << "Failed to retrieve the contents mime type of the collection: " << c;
     mParent->setCollection(Collection());
   }
 }
@@ -246,7 +249,6 @@ ItemModel::ItemModel( QObject *parent ) :
     QAbstractTableModel( parent ),
     d( new Private( this ) )
 {
-  setSupportedDragActions( Qt::MoveAction | Qt::CopyAction );
 }
 
 ItemModel::~ItemModel()
@@ -325,7 +327,7 @@ QVariant ItemModel::headerData( int section, Qt::Orientation orientation, int ro
 
 void ItemModel::setCollection( const Collection &collection )
 {
-  kDebug( 5250 );
+  kDebug();
   if ( d->collection == collection )
     return;
 
@@ -442,4 +444,10 @@ Collection ItemModel::collection() const
   return d->collection;
 }
 
+Qt::DropActions ItemModel::supportedDropActions() const
+{
+  return Qt::CopyAction | Qt::MoveAction | Qt::LinkAction;
+}
+
+
 #include "itemmodel.moc"
