@@ -1,20 +1,20 @@
 /*  -*- c++ -*-
     kmime_headers.cpp
 
-    KMime, the KDE internet mail/usenet news message library.
+    KMime, the KDE Internet mail/usenet news message library.
     Copyright (c) 2001-2002 the KMime authors.
     See file AUTHORS for details
     Copyright (c) 2006 Volker Krause <vkrause@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
-    License assert published by the Free Software Foundation; either
+    License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for morbe details.
+    Library General Public License for more details.
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
@@ -44,6 +44,7 @@
 #include "kmime_content.h"
 #include "kmime_codecs.h"
 #include "kmime_header_parsing.h"
+#include "kmime_headerfactory_p.h"
 #include "kmime_warning.h"
 
 #include <QtCore/QTextCodec>
@@ -56,6 +57,21 @@
 #include <assert.h>
 #include <ctype.h>
 
+template <typename T>
+bool registerHeaderHelper()
+{
+  const T dummy;
+  if( QByteArray( dummy.type() ).isEmpty() ) {
+    // This is a generic header.
+    return false;
+  }
+  return KMime::HeaderFactory::self()->registerHeader<T>();
+}
+
+// macro to register a header with HeaderFactory
+#define kmime_register_header( subclass )                             \
+namespace { const bool dummyForRegistering##subclass = registerHeaderHelper<subclass>(); }
+
 // macro to generate a default constructor implementation
 #define kmime_mk_trivial_ctor( subclass, baseclass )                  \
 subclass::subclass( Content *parent ) : baseclass( parent )           \
@@ -74,7 +90,10 @@ subclass::subclass( Content *parent, const QString &s, const QByteArray &charset
   fromUnicodeString( s, charset );                                    \
 }                                                                     \
                                                                       \
-subclass::~subclass() {}
+subclass::~subclass() {}                                              \
+                                                                      \
+kmime_register_header( subclass )
+// end kmime_mk_trivial_ctor
 
 
 #define kmime_mk_trivial_ctor_with_dptr( subclass, baseclass ) \
@@ -94,7 +113,11 @@ subclass::subclass( Content *parent, const QString &s, const QByteArray &charset
   fromUnicodeString( s, charset );                                    \
 }                                                                     \
                                                                       \
-subclass::~subclass() {}
+subclass::~subclass() {}                                              \
+                                                                      \
+kmime_register_header( subclass )
+// end kmime_mk_trivial_ctor_with_dptr
+
 
 #define kmime_mk_trivial_ctor_with_name( subclass, baseclass, name )  \
 kmime_mk_trivial_ctor( subclass, baseclass )                          \
@@ -136,6 +159,7 @@ Base::Base( BasePrivate *dd, KMime::Content *parent ) :
 Base::~Base()
 {
   delete d_ptr;
+  d_ptr = 0;
 }
 
 KMime::Content *Base::parent() const
@@ -1039,6 +1063,7 @@ void Ident::appendIdentifier( const QByteArray &id )
 
 //@cond PRIVATE
 kmime_mk_trivial_ctor_with_dptr( SingleIdent, Ident )
+kmime_mk_dptr_ctor( SingleIdent, Ident )
 //@endcond
 
 QByteArray SingleIdent::identifier() const
@@ -1159,6 +1184,8 @@ bool ReturnPath::parse( const char* &scursor, const char * const send,
 
 //-----<Generic>-------------------------------
 
+// NOTE: Do *not* register Generic with HeaderFactory, since its type() is changeable.
+
 Generic::Generic() : Generics::Unstructured( new GenericPrivate )
 {
 }
@@ -1411,7 +1438,7 @@ bool MailCopiesTo::parse( const char *& scursor, const char * const send,
       return true;
     }
     if ( qstrnicmp( "nobody", scursor, 6 ) == 0 ) {
-      d->alwaysCopy = true;
+      d->neverCopy = true;
       return true;
     }
   }
@@ -1897,6 +1924,91 @@ success:
 
 //-----</Content-Type>-------------------------
 
+//-----<ContentID>----------------------
+
+kmime_mk_trivial_ctor_with_name_and_dptr( ContentID, SingleIdent, Content-ID )
+kmime_mk_dptr_ctor( ContentID, SingleIdent )
+
+bool ContentID::parse( const char* &scursor, const char *const send, bool isCRLF )
+{
+  Q_D ( ContentID );
+  // Content-id := "<" contentid ">"
+  // contentid := now whitespaces
+
+  const char* origscursor = scursor;
+  if ( !SingleIdent::parse ( scursor, send, isCRLF ) )
+  {
+    scursor = origscursor;
+    d->msgIdList.clear();
+
+    while ( scursor != send )
+    {
+      eatCFWS ( scursor, send, isCRLF );
+      // empty entry ending the list: OK.
+      if ( scursor == send )
+      {
+        return true;
+      }
+      // empty entry: ignore.
+      if ( *scursor == ',' )
+      {
+        scursor++;
+        continue;
+      }
+
+      AddrSpec maybeContentId;
+      // Almost parseAngleAddr
+      if ( scursor == send || *scursor != '<' )
+      {
+        return false;
+      }
+      scursor++; // eat '<'
+
+      eatCFWS ( scursor, send, isCRLF );
+      if ( scursor == send )
+      {
+        return false;
+      }
+
+      // Save chars untill '>''
+      QString result = "";
+      if( !parseAtom(scursor, send, result, false) ) {
+        return false;
+      }
+
+      eatCFWS ( scursor, send, isCRLF );
+      if ( scursor == send || *scursor != '>' )
+      {
+        return false;
+      }
+      scursor++;
+      // /Almost parseAngleAddr
+
+      maybeContentId.localPart = result;
+      d->msgIdList.append ( maybeContentId );
+
+      eatCFWS ( scursor, send, isCRLF );
+      // header end ending the list: OK.
+      if ( scursor == send )
+      {
+        return true;
+      }
+      // regular item separator: eat it.
+      if ( *scursor == ',' )
+      {
+        scursor++;
+      }
+    }
+    return true;
+  }
+  else
+  {
+    return true;
+  }
+}
+
+//-----</ContentID>----------------------
+
 //-----<ContentTransferEncoding>----------------------------
 
 //@cond PRIVATE
@@ -2113,7 +2225,6 @@ kmime_mk_trivial_ctor_with_name( Bcc, Generics::AddressList, Bcc )
 kmime_mk_trivial_ctor_with_name( ReplyTo, Generics::AddressList, Reply-To )
 kmime_mk_trivial_ctor_with_name( Keywords, Generics::PhraseList, Keywords )
 kmime_mk_trivial_ctor_with_name( MIMEVersion, Generics::DotAtom, MIME-Version )
-kmime_mk_trivial_ctor_with_name( ContentID, Generics::SingleIdent, Content-ID )
 kmime_mk_trivial_ctor_with_name( Supersedes, Generics::SingleIdent, Supersedes )
 kmime_mk_trivial_ctor_with_name( InReplyTo, Generics::Ident, In-Reply-To )
 kmime_mk_trivial_ctor_with_name( References, Generics::Ident, References )
