@@ -26,10 +26,13 @@ class KMimeContentTest : public QObject
   Q_OBJECT
   private Q_SLOTS:
     void testGetHeaderInstance();
-    void testSetContent();
-    void testMultipartMixed();
+    void testHeaderAddRemove();
+    void testHeaderAppendPrepend();
     void testImplicitMultipartGeneration();
     void testExplicitMultipartGeneration();
+    void testSetContent();
+    void testEncodedContent();
+    void testMultipartMixed();
     void testMultipleHeaderExtraction();
     /**
       Tests that a message with uuencoded content
@@ -37,7 +40,9 @@ class KMimeContentTest : public QObject
       MIME structure is created.
     */
     void testParsingUuencoded();
+    // TODO: grab samples from http://www.yenc.org/develop.htm and make a Yenc test
     void testParent();
+    void testFreezing();
 };
 
 #endif
